/*
 * Copyright (c) 2026 Red Hat.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 */

#ifndef _CLUSTERS_H
#define _CLUSTERS_H

#define BTRFS_INFO_CLUST	 0
#define BTRFS_COMMIT_CLUST	 1
#define BTRFS_ALLOC_DATA_CLUST	 2
#define BTRFS_ALLOC_META_CLUST	 3
#define BTRFS_ALLOC_SYS_CLUST	 4
#define BTRFS_ALLOC_GLOBAL_CLUST 5
#define BTRFS_DEV_CLUST		 6
#define BTRFS_DISCARD_CLUST	 7

#endif /* _CLUSTERS_H */
